# Load the data
smoke<-read.table('C:/GRE2016/Data/nlsy_smoke_wide.dat', na.string='.')

# Give the variable names
names(smoke)<-c('id', 'smk15', 'smk16', 'smk17', 'smk18', 'smk19', 'smk20', 
                          'c_smk15', 'c_smk16', 'c_smk17', 'c_smk18', 'c_smk19', 'c_smk20',
                          'g_smk15', 'g_smk16', 'g_smk17', 'g_smk18', 'g_smk19', 'g_smk20')

library(OpenMx)

#Converting Items of Interest to Categorical Variables
smoke$c_smk15 <- mxFactor(smoke$c_smk15, levels=c(0,1))
smoke$c_smk16 <- mxFactor(smoke$c_smk16, levels=c(0,1))
smoke$c_smk17 <- mxFactor(smoke$c_smk17, levels=c(0,1))
smoke$c_smk18 <- mxFactor(smoke$c_smk18, levels=c(0,1))
smoke$c_smk19 <- mxFactor(smoke$c_smk19, levels=c(0,1))
smoke$c_smk20 <- mxFactor(smoke$c_smk20, levels=c(0,1))

################# Linear Growth Model ################# 
lgm.smk.omx <- mxModel("Dichotomous Linear Growth Model, Path Specification", type="RAM",
 mxData(observed=smoke, type="raw"),
 manifestVars=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 latentVars=c('eta_1','eta_2'),

# Residual Variances
mxPath(from=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=2, free=FALSE, values=1),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(1,0,1), labels=c('psi_11','psi_21','psi_22')),

# Factor Loadings 
mxPath(from='eta_1', to=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=1, free=FALSE, values=c(0,1,2,3,4,5)),

# Latent Variable Means  
mxPath(from='one', to='eta_2',
 arrows=1, free=TRUE, values=.2, labels='alpha_2'),

# Thresholds
mxThreshold(vars=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 nThresh=1, free=TRUE, values=2, labels='tau')

) # Close Model

lgm.smk.fit <- mxRun(lgm.smk.omx)
summary(lgm.smk.fit)

#free parameters:
#     name     matrix   row     col   Estimate  Std.Error
#1  psi_11          S eta_1   eta_1 2.57484598 0.43668042
#2  psi_21          S eta_1   eta_2 0.06447125 0.06131143
#3  psi_22          S eta_2   eta_2 0.11421569 0.02567498
#4 alpha_2          M     1   eta_2 0.22261005 0.03349526
#5     tau Thresholds     1 c_smk15 2.53124781 0.13447231
#
#observed statistics:  17112 
#estimated parameters:  5 
#degrees of freedom:  17107 
#-2 log likelihood:  13643.59 
#number of observations:  7192 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -20570.41               13653.59                       NA
#BIC:     -138278.96               13688.00                 13672.11
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-09-25 10:52:04 
#wall clock time: 31.71017 secs 
#OpenMx version number: 2.0.0.3751 
#Need help?  See help(mxSummary) 


# Dichotomous Outcome - Linear Growth Model - Constraining total variance to be 1

lgm.smk.omx <- mxModel("Dichotomous Linear Growth Model, Path Specification",
 type="RAM", mxData(observed=smoke, type="raw"),
 manifestVars=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 latentVars=c('eta_1','eta_2'),

# Residual Variances 
mxPath(from=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=2, free=FALSE, labels=c('th[1,1]','th[2,1]','th[3,1]','th[4,1]','th[5,1]','th[6,1]')),

# Latent Variable Covariances
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(1,0,1), labels=c('psi_1','psi_21','psi_2')),

# Factor Loadings
mxPath(from='eta_1', to=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 arrows=1, free=FALSE, values=c(0,1,2,3,4,5)),

# Latent Variable Means 
mxPath(from='one', to='eta_2', arrows=1,
 free=TRUE, values=.2, labels='alpha_2'),

# Thresholds
mxThreshold(vars=c('c_smk15','c_smk16','c_smk17','c_smk18','c_smk19','c_smk20'),
 nThresh=1, free=TRUE, values=2, labels='tau'),

mxMatrix('Full', 6, 1, free = FALSE, values = c(0,1,2,3,4,5), name = 'time'),
mxMatrix('Full', 6, 1, free = TRUE, values = 1, labels = 'psi_1',  name = 'var_1'),
mxMatrix('Full', 6, 1, free = TRUE, values = 0, labels = 'psi_21', name = 'cov_21'),
mxMatrix('Full', 6, 1, free = TRUE, values = 1, labels = 'psi_2',  name = 'var_2'),

mxMatrix('Full', 6, 1, free = FALSE, values = 2, name = 'two'),
mxMatrix('Full', 6, 1, free = FALSE, values = 1, name = 'one'),

mxAlgebra(one - var_1+time*var_2*time+two*cov_21*time, name='th')

) # Close Model
 
lgm.smk.fit <- mxRun(lgm.smk.omx)

summary(lgm.smk.fit)


#free parameters:
#     name     matrix   row     col     Estimate   Std.Error
#1   psi_1          S eta_1   eta_1  0.814919412 0.028719470
#2  psi_21          S eta_1   eta_2 -0.008334899 0.007608922
#3   psi_2          S eta_2   eta_2  0.015227785 0.004541346
#4 alpha_2          M     1   eta_2  0.131241204 0.012791148
#5     tau Thresholds     1 c_smk15  1.311963896 0.025538597
#
#observed statistics:  17112 
#estimated parameters:  5 
#degrees of freedom:  17107 
#-2 log likelihood:  13668.63 
#number of observations:  7192 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -20545.37               13678.63                       NA
#BIC:     -138253.93               13713.03                 13697.14
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-09-25 12:08:37 
#wall clock time: 39.75697 secs 
#OpenMx version number: 2.0.0.3751 
#Need help?  See help(mxSummary) 


###### Polytomous Linear Growth Model ######

#Converting Items of Interest to Categorical Variables
smoke$g_smk15 <- mxFactor(smoke$g_smk15, levels=c(0,1,2,3))
smoke$g_smk16 <- mxFactor(smoke$g_smk16, levels=c(0,1,2,3))
smoke$g_smk17 <- mxFactor(smoke$g_smk17, levels=c(0,1,2,3))
smoke$g_smk18 <- mxFactor(smoke$g_smk18, levels=c(0,1,2,3))
smoke$g_smk19 <- mxFactor(smoke$g_smk19, levels=c(0,1,2,3))
smoke$g_smk20 <- mxFactor(smoke$g_smk20, levels=c(0,1,2,3))

################# Linear Growth Model ################# 
lgm.g_smk.omx <- mxModel("Polytomous Linear Growth Model, Path Specification", type="RAM",
 mxData(observed=smoke, type="raw"),
 manifestVars=c('g_smk15','g_smk16','g_smk17','g_smk18','g_smk19','g_smk20'),
 latentVars=c('eta_1','eta_2'),

# Residual Variances
mxPath(from=c('g_smk15','g_smk16','g_smk17','g_smk18','g_smk19','g_smk20'),
 arrows=2, free=FALSE, values=1),

# Latent Variable Covariances 
mxPath(from=c('eta_1','eta_2'), connect='unique.pairs',
 arrows=2, free=TRUE, values = c(1,0,1), labels=c('psi_11','psi_21','psi_22')),

# Factor Loadings
mxPath(from='eta_1', to=c('g_smk15','g_smk16','g_smk17','g_smk18','g_smk19','g_smk20'),
 arrows=1, free=FALSE, values=1),

mxPath(from='eta_2', to=c('g_smk15','g_smk16','g_smk17','g_smk18','g_smk19','g_smk20'),
 arrows=1, free=FALSE, values=c(0,1,2,3,4,5)),

# Latent Variable Mean
mxPath(from='one', to='eta_2',
 arrows=1, free=TRUE, values=.2, labels='alpha_2'),

# Thresholds
mxThreshold(vars=c('g_smk15','g_smk16','g_smk17','g_smk18','g_smk19','g_smk20'),
 nThresh=3, free=TRUE, values=c(1,2,3), labels=c('tau1','tau2','tau3'))

) # Close Model   

lgm.g_smk.fit <- mxRun(lgm.g_smk.omx)

summary(lgm.g_smk.fit)

#free parameters:
#     name     matrix   row     col    Estimate  Std.Error
#1  psi_11          S eta_1   eta_1  2.85859353 0.25061142
#2  psi_21          S eta_1   eta_2 -0.02188046 0.03732851
#3  psi_22          S eta_2   eta_2  0.12619314 0.01606315
#4 alpha_2          M     1   eta_2  0.22325594 0.01391316
#5    tau1 Thresholds     1 g_smk15  1.35424698 0.05155838
#6    tau2 Thresholds     2 g_smk15  1.92062841 0.05899488
#7    tau3 Thresholds     3 g_smk15  2.53337005 0.06792426
#
#observed statistics:  17112 
#estimated parameters:  7 
#degrees of freedom:  17105 
#-2 log likelihood:  30304.58 
#number of observations:  7192 
#Information Criteria: 
#      |  df Penalty  |  Parameters Penalty  |  Sample-Size Adjusted
#AIC:      -3905.417               30318.58                       NA
#BIC:    -121600.211               30366.75                  30344.5
#Some of your fit indices are missing.
#  To get them, fit saturated and independence models, and include them with
#  summary(yourModel, SaturatedLikelihood=..., IndependenceLikelihood=...). 
#timestamp: 2014-09-25 12:43:18 
#wall clock time: 46.36064 secs 
#OpenMx version number: 2.0.0.3751 
#Need help?  See help(mxSummary) 