smoke = read.table('C:\\GRE2016\\Data\\nlsy_smoke_long.dat', na.strings='.')

names(smoke) = c('id','age','smoke','c_smk','g_smk')


#### mixor gives estimates that were consistent with Mplus and NLMIXED, lme4 did not

#install.packages('mixor')

library(mixor)


### Linear growth model for dichotomous outcome
smoke$age_c15 = smoke$age - 15

lg.smk.1 = mixor(c_smk~age_c15, data=smoke, id=id, which.random.slope = 1, link="logit")

summary(lg.smk.1)

#Call:
#mixor(formula = c_smk ~ age_c15, data = smoke, id = id, which.random.slope = 1, 
#    link = "logit")
#
#Deviance =         13644.95 
#Log-likelihood =  -6822.475 
#RIDGEMAX =         0.2 
#AIC =             -6827.475 
#SBC =             -6844.677 
#
#                         Estimate Std. Error  z value   P(>|z|)    
#(Intercept)             -4.457997   0.214158 -20.8164 < 2.2e-16 ***
#age_c15                  0.402710   0.055417   7.2669 3.679e-13 ***
#(Intercept) (Intercept)  7.843451   1.162361   6.7479 1.500e-11 ***
#(Intercept) age_c15      0.254488   0.167447   1.5198 0.1285577    
#age_c15 age_c15          0.288756   0.078284   3.6886 0.0002255 ***
#---
#Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1


library(lme4)

lg.smk.2 = glmer(c_smk ~ 1 + age_c15 + (1 + age_c15|id), data = smoke, family = 'binomial')

summary(lg.smk.2)

#Generalized linear mixed model fit by maximum likelihood (Laplace
#  Approximation) [glmerMod]
# Family: binomial  ( logit )
#Formula: c_smk ~ 1 + age_c15 + (1 + age_c15 | id)
#   Data: smoke
#
#     AIC      BIC   logLik deviance df.resid 
# 10609.8  10648.5  -5299.9  10599.8    17107 
#
#Scaled residuals: 
#     Min       1Q   Median       3Q      Max 
#-1.37900 -0.00185 -0.00043 -0.00010  1.44927 
#
#Random effects:
# Groups Name        Variance Std.Dev. Corr 
# id     (Intercept) 630.2    25.1          
#        age_c15     156.2    12.5     -0.43
#Number of obs: 17112, groups:  id, 7192
#
#Fixed effects:
#            Estimate Std. Error z value Pr(>|z|)    
#(Intercept) -18.3763     0.8015 -22.926   <2e-16 ***
#age_c15       1.4485     0.1635   8.862   <2e-16 ***
#---
#Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1
#
#Correlation of Fixed Effects:
#        (Intr)
#age_c15 -0.965


lg.smk.3 = glmer(c_smk ~ 1 + age_c15 + (1 + age_c15|id), data = smoke, family = binomial, nAGQ = 0)

summary(lg.smk.3)

#Generalized linear mixed model fit by maximum likelihood (Adaptive
#  Gauss-Hermite Quadrature, nAGQ = 0) [glmerMod]
# Family: binomial  ( logit )
#Formula: c_smk ~ 1 + age_c15 + (1 + age_c15 | id)
#   Data: smoke
#
#     AIC      BIC   logLik deviance df.resid 
# 14123.7  14162.4  -7056.8  14113.7    17107 
#
#Scaled residuals: 
#    Min      1Q  Median      3Q     Max 
#-1.6504 -0.3133 -0.2223 -0.1481  2.8004 
#
#Random effects:
# Groups Name        Variance Std.Dev. Corr
# id     (Intercept) 4.786451 2.18780      
#        age_c15     0.003048 0.05521  1.00
#Number of obs: 17112, groups:  id, 7192
#
#Fixed effects:
#            Estimate Std. Error z value Pr(>|z|)    
#(Intercept) -3.02343    0.06051  -49.96   <2e-16 ***
#age_c15      0.39481    0.01634   24.17   <2e-16 ***
#---
#Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1
#
#Correlation of Fixed Effects:
#        (Intr)
#age_c15 -0.757


### Linear Growth Model for Ordinal Outcome

lg.smk.4 = mixor(g_smk~age_c15, data=smoke, id=id, which.random.slope = 1, link="logit")

summary(lg.smk.4)

#Call:
#mixor(formula = g_smk ~ age_c15, data = smoke, id = id, which.random.slope = 1, 
#    link = "logit")
#
#Deviance =         30309.36 
#Log-likelihood =  -15154.68 
#RIDGEMAX =         0.2 
#AIC =             -15161.68 
#SBC =             -15185.76 
#
#                         Estimate Std. Error  z value   P(>|z|)    
#(Intercept)             -2.389477   0.096436 -24.7779 < 2.2e-16 ***
#age_c15                  0.396773   0.026749  14.8332 < 2.2e-16 ***
#(Intercept) (Intercept)  9.105819   0.805301  11.3073 < 2.2e-16 ***
#(Intercept) age_c15     -0.105570   0.121618  -0.8680    0.3854    
#age_c15 age_c15          0.403255   0.052407   7.6946 1.421e-14 ***
#Threshold1               1.003906   0.029392  34.1559 < 2.2e-16 ***
#Threshold2               2.094453   0.052206  40.1190 < 2.2e-16 ***
#---
#Signif. codes:  0 *** 0.001 ** 0.01 * 0.05 . 0.1   1


## Note: Threshold1 and Threshold2 add to the (Intercept)

