ODS HTML CLOSE;
ODS LISTING;

DATA smoke;
	INFILE 'C:\GRE2016\Data\nlsy_smoke_long.dat';
	INPUT id age smoke c_smk g_smk;
RUN;

PROC MEANS DATA = smoke;
	CLASS age;
	VAR c_smk;
RUN;

* Dichotomous Growth Model;
PROC NLMIXED DATA=smoke;
    theta_ti = b_1i + b_2i * (age-15);

    p = EXP(theta_ti-omega)/(1+EXP(theta_ti-omega));

    MODEL c_smk ~ BINARY(p);
    RANDOM b_1i b_2i ~ NORMAL([0, beta_2],[v_1, c_21, v_2]) SUBJECT=id;

    PARMS 
       omega=4    beta_2=.5    v_1=6    c_21=0    v_2=.5;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                  13637
                            AIC (smaller is better)            13647
                            AICC (smaller is better)           13647
                            BIC (smaller is better)            13682


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  omega        4.7115    0.2591  7190    18.19    <.0001    0.05    4.2036    5.2193  0.003914
  beta_2       0.4261   0.06116  7190     6.97    <.0001    0.05    0.3063    0.5460   -0.0229
  v_1          9.4332    1.5833  7190     5.96    <.0001    0.05    6.3294   12.5369  -0.00017
  c_21         0.1232    0.2135  7190     0.58    0.5640    0.05   -0.2954    0.5418  -0.00586
  v_2          0.3880   0.09112  7190     4.26    <.0001    0.05    0.2094    0.5666  -0.00323
*/

* Determining probabilities of responding in each category;
DATA sim_logit;
  omega   = 4.7115;
  beta_2  = 0.4261;
  v_1     = 9.4332;
  c_21    = 0.1232;
  v_2     = 0.3880;

  * Distribution of theta_ti at each point in time and probabilities;

	DO t = 0 to 5;
  		mean_theta = 0 + beta_2 * t;
		var_theta = v_1 + t*v_2*t + 2*c_21*t + (constant('pi')**2)/3;
		std_theta = SQRT(var_theta);

		p2 = probnorm((mean_theta - (omega))/std_theta);

	OUTPUT;
	END;
RUN;
PROC PRINT;
RUN;


* Dichotomous Growth Model - Alternative Specification;
PROC NLMIXED DATA=smoke;
    theta_ti = b_1i + b_2i * (age-15);

    p = EXP(alpha * (theta_ti-omega))/(1+EXP(alpha * (theta_ti-omega)));

    MODEL c_smk ~ BINARY(p);
    RANDOM b_1i b_2i ~ NORMAL([0, beta_2],[1, c_21, v_2]) SUBJECT=id;

    PARMS 
       omega=1.5    beta_2=.1    alpha=3    c_21=0    v_2=.1;
RUN;
/*
                                         Fit Statistics

                            -2 Log Likelihood                  13643
                            AIC (smaller is better)            13653
                            AICC (smaller is better)           13653
                            BIC (smaller is better)            13687


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  omega        1.5663   0.05496  7190    28.50    <.0001    0.05    1.4586    1.6740  -0.00031
  beta_2       0.1417   0.01299  7190    10.90    <.0001    0.05    0.1162    0.1671  -0.00353
  alpha        2.9057    0.2235  7190    13.00    <.0001    0.05    2.4675    3.3438  -0.00061
  c_21        0.02283   0.02472  7190     0.92    0.3558    0.05  -0.02562   0.07128  -0.00486
  v_2         0.03982  0.009891  7190     4.03    <.0001    0.05   0.02043   0.05921  0.007332
*/



* Dichotomous Growth Model - Alternative Specification;
PROC NLMIXED DATA=smoke;
	theta_ti = b_1i + b_2i * (age - 15);

	p = exp(theta_ti)/(1+exp(theta_ti));

	MODEL c_smk ~ BINARY(p);
	RANDOM b_1i b_2i ~ NORMAL([beta_1, beta_2],[v_1, c_21, v_2]) SUBJECT=id;

	PARMS beta_1 = -4 beta_2 = .4 v_1 = 8 c_21 = 0 v_2 = .5;
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                  13631
                            AIC (smaller is better)            13641
                            AICC (smaller is better)           13641
                            BIC (smaller is better)            13675


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  beta_1      -4.7172    0.2390  7190   -19.74    <.0001    0.05   -5.1858   -4.2487  -0.00179
  beta_2       0.4169   0.06002  7190     6.95    <.0001    0.05    0.2993    0.5346  -0.00176
  v_1          9.3496    1.3959  7190     6.70    <.0001    0.05    6.6133   12.0859  -0.00037
  c_21         0.1347    0.1990  7190     0.68    0.4985    0.05   -0.2554    0.5248  -0.00061
  v_2          0.4130   0.07498  7190     5.51    <.0001    0.05    0.2660    0.5600  -0.00036
*/

* Dichotomous Model - Probit Link;
PROC NLMIXED DATA = smoke;
    ys_ti = b_1i + b_2i*(age-15);

    p = probnorm(ys_ti-tau);

    MODEL c_smk ~ BINARY(p);
    RANDOM b_1i b_2i ~ normal([0,beta_2],[v_1, c_21, v_2]) SUBJECT = id;

    PARMS 
        tau=-2  beta_2=.3  v_1=3  c_21=0  v_2=.2; 
RUN;

/*
                                         Fit Statistics

                            -2 Log Likelihood                  13641
                            AIC (smaller is better)            13651
                            AICC (smaller is better)           13651
                            BIC (smaller is better)            13686


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  tau          2.5677    0.1296  7190    19.81    <.0001    0.05    2.3136    2.8218  -0.00068
  beta_2       0.2313   0.03219  7190     7.18    <.0001    0.05    0.1682    0.2944  0.006401
  v_1          2.6992    0.4250  7190     6.35    <.0001    0.05    1.8661    3.5323  -0.00003
  c_21        0.04953   0.05994  7190     0.83    0.4087    0.05  -0.06798    0.1670  0.002201
  v_2          0.1142   0.02563  7190     4.46    <.0001    0.05   0.06399    0.1645  0.001564
*/


* Determining probabilities of responding in each category;
DATA sim_probit;
  tau    =  2.5677;
  beta_2 =  0.2313;
  v_1    =  2.6992;
  c_21   = 0.04953;
  v_2    =  0.1142;

  * Distribution of theta_ti at each point in time and probabilities;

	DO t = 0 to 5;
  		mean_ys = 0 + beta_2 * t;
		var_ys = v_1 + t*v_2*t + 2*c_21*t + 1;
		std_ys = SQRT(var_ys);

		p2 = probnorm((mean_ys - tau)/std_ys);

	OUTPUT;
	END;
RUN;
PROC PRINT;
RUN;



* Polytomous Growth Model with logit link;
PROC NLMIXED DATA = smoke;
	theta_ti = b_1i+b_2i*(age-15);

	e1=(theta_ti-omega);
	e2=(theta_ti-(omega+delta_1));
	e3=(theta_ti-(omega+delta_1+delta_2));

	IF      g_smk=0 THEN p=1-EXP(e1)/(1+EXP(e1));
	ELSE IF g_smk=1 THEN p=EXP(e1)/(1+EXP(e1))-EXP(e2)/(1+EXP(e2));
	ELSE IF g_smk=2 THEN p=EXP(e2)/(1+EXP(e2))-EXP(e3)/(1+EXP(e3));
	ELSE IF g_smk=3 THEN p=EXP(e3)/(1+EXP(e3));

	IF (p > 1E-8) THEN ll = LOG(p);
	ELSE ll = -1E100;

	MODEL g_smk ~ GENERAL(ll);
	RANDOM b_1i b_2i ~ NORMAL([0,beta_2],[v_1, c_21, v_2]) SUBJECT = id;

	PARMS 	
      omega = 2  delta_1 = 1  delta_2 = 1  beta_2 = .4
      v_1  = 8   c_21  = 0    v_2  = .5;
	BOUNDS
      delta_1-delta_2 > 0;
RUN;


/*
                                        Fit Statistics

                            -2 Log Likelihood                  30308
                            AIC (smaller is better)            30322
                            AICC (smaller is better)           30322
                            BIC (smaller is better)            30370


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  omega        2.4046   0.09663  7190    24.88    <.0001    0.05    2.2151    2.5940  0.000024
  delta_1      1.0045   0.03111  7190    32.29    <.0001    0.05    0.9435    1.0654  0.000687
  delta_2      1.0909   0.03472  7190    31.41    <.0001    0.05    1.0228    1.1589  0.000281
  beta_2       0.3961   0.02619  7190    15.12    <.0001    0.05    0.3448    0.4475  0.000726
  v_1          9.1870    0.8163  7190    11.25    <.0001    0.05    7.5868   10.7872  -0.00005
  c_21        -0.1087    0.1226  7190    -0.89    0.3749    0.05   -0.3490    0.1315   0.00011
  v_2          0.4025   0.05044  7190     7.98    <.0001    0.05    0.3037    0.5014  -0.00026
*/

* Determining probabilities of responding in each category;
DATA sim_logit;
  omega   =  2.4046; 
  delta_1 =  1.0045; 
  delta_2 =  1.0909; 
  beta_2  =  0.3961; 
  v_1     =  9.1870; 
  c_21    = -0.1087; 
  v_2     =  0.4025; 

  * Distribution of theta_ti at each point in time and probabilities;

	DO t = 0 to 5;
  		mean_theta = 0 + beta_2 * t;
		var_theta = v_1 + t*v_2*t + 2*c_21*t + constant('pi')**2/3;
		std_theta = SQRT(var_theta);

		ps2 = probnorm((mean_theta - (omega))/std_theta);
		ps3 = probnorm((mean_theta - (omega+delta_1))/std_theta);
		ps4 = probnorm((mean_theta - (omega+delta_1+delta_2))/std_theta);

		p1 = 1 - ps2;
		p2 = ps2 - ps3;
		p3 = ps3 - ps4;
		p4 = ps4;
	OUTPUT;
	END;
RUN;
PROC PRINT;
RUN;

	
* Polytomous Growth Model with probit link;
PROC NLMIXED DATA = smoke;
   ys_ti = b_1i + b_2i*(age-15);

   IF      g_smk=0 THEN p = 1 - probnorm(ys_ti-tau1);
   ELSE IF g_smk=1 THEN p = probnorm(ys_ti-tau1) - probnorm(ys_ti-(tau1+tau2));
   ELSE IF g_smk=2 THEN p = probnorm(ys_ti-(tau1+tau2)) - probnorm(ys_ti-(tau1+tau2+tau3));
   ELSE IF g_smk=3 THEN p = probnorm(ys_ti-(tau1+tau2+tau3));

   IF (p > 1E-8) THEN ll = log(p);
   ELSE ll = -1E100;

   MODEL g_smk ~ general(ll);
   RANDOM b_1i b_2i ~ NORMAL([0,beta_2], [v_1, c_21, v_2]) SUBJECT = id;

   PARMS tau1=1 tau2=1 tau3=1 beta_2=.5 v_1=2 c_21=0 v_2=.2;
   BOUNDS tau2-tau3 > 0; 
RUN; 


/*
                                         Fit Statistics

                            -2 Log Likelihood                  30304
                            AIC (smaller is better)            30318
                            AICC (smaller is better)           30318
                            BIC (smaller is better)            30367


                                      Parameter Estimates

                       Standard
  Parameter  Estimate     Error    DF  t Value  Pr > |t|   Alpha     Lower     Upper  Gradient

  tau1         1.3553   0.05271  7190    25.71    <.0001    0.05    1.2519    1.4586  -0.00039
  tau2         0.5665   0.01715  7190    33.03    <.0001    0.05    0.5329    0.6001  -0.00689
  tau3         0.6129   0.01914  7190    32.03    <.0001    0.05    0.5754    0.6504  -0.00242
  beta_2       0.2233   0.01450  7190    15.40    <.0001    0.05    0.1949    0.2518  0.005752
  v_1          2.8614    0.2533  7190    11.30    <.0001    0.05    2.3648    3.3579  0.000175
  c_21       -0.02151   0.03868  7190    -0.56    0.5782    0.05  -0.09732   0.05431  0.001533
  v_2          0.1263   0.01600  7190     7.89    <.0001    0.05   0.09493    0.1577  0.006611
*/

* Determining probabilities of responding in each category;
DATA sim_probit;
  tau1   =  1.3553; 
  tau2   =  0.5665; 
  tau3   =  0.6129; 
  beta_2 =  0.2233; 
  v_1    =  2.8614; 
  c_21   = -0.02151;
  v_2    =  0.1263;

  * Distribution of theta_ti at each point in time and probabilities;

	DO t = 0 to 5;
  		mean_ys = 0 + beta_2 * t;
		var_ys  = v_1 + t*v_2*t + 2*c_21*t + 1;
		std_ys  = SQRT(var_ys);

		ps2 = probnorm((mean_ys - (tau1))/std_ys);
		ps3 = probnorm((mean_ys - (tau1+tau2))/std_ys);
		ps4 = probnorm((mean_ys - (tau1+tau2+tau3))/std_ys);

		p1 = 1 - ps2;
		p2 = ps2 - ps3;
		p3 = ps3 - ps4;
		p4 = ps4;
	OUTPUT;
	END;
RUN;
PROC PRINT;
RUN;

